package com.me.common.ecs;

import java.util.HashMap;
import java.util.Map;

public class ComponentType {

    private static final Map<Class<? extends Component>, ComponentType> types = new HashMap<>();
    private static final ComponentType[] typeById = new ComponentType[Long.SIZE];

    private static long nextBit = 1l;
    private static int nextId = 0;

    private long bits;
    private int id;

    private ComponentType() {
        this.bits = nextBit;
        this.id = nextId++;

        this.nextBit <<= 1;
    }

    protected long getBits() {
        return bits;
    }

    protected int getId() {
        return this.id;
    }

    protected static long getMaskBits(Class<? extends Component>... components) {
        long mask = 0l;
        for (Class<? extends Component> clazz : components) {
            mask |= getTypeBits(clazz);
        }
        return mask;
    }

    protected static long getTypeBits(Class<? extends Component> component) {
        return getComponentType(component).getBits();
    }

    protected static int getTypeId(Class<? extends Component> component) {
        return getComponentType(component).getId();
    }

    protected static ComponentType getById(int id) {
        return typeById[id];
    }

    protected boolean isTypeInMask(long mask) {
        return (bits & mask) == mask;
    }

    protected static void registerComponentType(Class<? extends Component> component) {
        ComponentType type = types.get(component);
        if (type != null) {
            throw new IllegalArgumentException(component.getName() + " has already been registered.");
        }
        type = new ComponentType();
        types.put(component, type);
        typeById[type.getId()] = type;
    }

    protected static ComponentType getComponentType(Class<? extends Component> component) {
        ComponentType type = types.get(component);
        if (type == null) {
            throw new IllegalArgumentException(component.getName() + " has not been registered.");
        }
        return type;
    }

    protected static int getRegisteredComponentTypeCount() {
        return types.size();
    }

}
